import unittest

from ndk.definitions import (CommandDirective, ContactDirective,
                             ContactGroupDirective, TimePeriodDirective)
from ndk.options.contact import HostNotifications, ServiceNotifications
from ndk.stack import Stack


class ContactDirectiveTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('ContactTesting')
        self.tp = TimePeriodDirective(
            self.stack, timeperiod_name='24x7', alias='foo')
        self.cmd = CommandDirective(
            self.stack, command_name='email', command_line='bar')
        self.requried_directive = dict(
            contact_name='Foo Bar',
            # contactgroups=self.cg,
            host_notifications_enabled=True,
            service_notifications_enabled=True,
            host_notifications_period=self.tp,
            service_notifications_period=self.tp,
            host_notification_commands=self.cmd,
            service_notification_commands=self.cmd,
            host_notifications_options=HostNotifications.all(),
            service_notifications_options=ServiceNotifications.empty(),
        )

    def test_contact_directive(self):
        with self.assertRaises(TypeError):
            ContactDirective(self.stack)
        assert ContactDirective(self.stack, **self.requried_directive)

    def test_pk_and_name(self):
        self.requried_directive.pop('contact_name')
        with self.assertRaises(TypeError):
            ContactDirective(self.stack, **self.requried_directive)

        self.requried_directive['contact_name'] = 'Foo Bar'
        contact = ContactDirective(self.stack, **self.requried_directive)
        assert contact.pk == 'foo-bar'
        assert contact.contact_name == 'foo-bar'

    def test_alias(self):
        contact = ContactDirective(self.stack, **self.requried_directive)
        assert contact.alias is None

        contact = ContactDirective(
            self.stack, alias='Foo Bar', **self.requried_directive)
        assert contact.alias == 'Foo Bar'

    def test_hostgroups(self):
        contact = ContactDirective(self.stack, **self.requried_directive)
        assert contact.contactgroups is None

        cg = ContactGroupDirective(
            self.stack, contactgroup_name='baz', alias='cg')
        contact = ContactDirective(
            self.stack, contactgroups=[cg], **self.requried_directive)
        assert isinstance(contact.contactgroups, list)
        assert isinstance(contact.contactgroups[0], ContactGroupDirective)

    def test_minimum_importance(self):
        contact = ContactDirective(self.stack, **self.requried_directive)
        assert contact.minimum_importance is None

        tp = ContactDirective(
            self.stack, minimum_importance=30, **self.requried_directive)
        assert isinstance(tp.minimum_importance, int)
        assert tp.minimum_importance == 30

    def test_host_notifications_enabled(self):
        contact = ContactDirective(self.stack, **self.requried_directive)
        assert contact.host_notifications_enabled

        self.requried_directive.pop('host_notifications_enabled')
        with self.assertRaises(TypeError):
            ContactDirective(self.stack, **self.requried_directive)

    def test_service_notifications_enabled(self):
        contact = ContactDirective(self.stack, **self.requried_directive)
        assert contact.service_notifications_enabled

        self.requried_directive.pop('service_notifications_enabled')
        with self.assertRaises(TypeError):
            ContactDirective(self.stack, **self.requried_directive)

    def test_host_notifications_period(self):
        contact = ContactDirective(self.stack, **self.requried_directive)
        assert contact.host_notifications_period is self.tp

        self.requried_directive.pop('host_notifications_period')
        with self.assertRaises(TypeError):
            ContactDirective(self.stack, **self.requried_directive)

    def test_service_notifications_period(self):
        contact = ContactDirective(self.stack, **self.requried_directive)
        assert contact.service_notifications_period is self.tp

        self.requried_directive.pop('service_notifications_period')
        with self.assertRaises(TypeError):
            ContactDirective(self.stack, **self.requried_directive)

    def test_host_notification_commands(self):
        contact = ContactDirective(self.stack, **self.requried_directive)
        assert contact.host_notification_commands is self.cmd

        self.requried_directive.pop('host_notification_commands')
        with self.assertRaises(TypeError):
            ContactDirective(self.stack, **self.requried_directive)

    def test_service_notification_commands(self):
        contact = ContactDirective(self.stack, **self.requried_directive)
        assert contact.service_notification_commands is self.cmd

        self.requried_directive.pop('service_notification_commands')
        with self.assertRaises(TypeError):
            ContactDirective(self.stack, **self.requried_directive)

    def test_host_notifications_options(self):
        contact = ContactDirective(self.stack, **self.requried_directive)
        assert contact.host_notifications_options == HostNotifications.all()

        self.requried_directive.pop('host_notifications_options')
        with self.assertRaises(TypeError):
            ContactDirective(self.stack, **self.requried_directive)

        self.requried_directive['host_notifications_options'] = [
            HostNotifications.DOWN,
            HostNotifications.UNREACHABLE
        ]
        contact = ContactDirective(self.stack, **self.requried_directive)
        assert contact.host_notifications_options == [
            HostNotifications.DOWN,
            HostNotifications.UNREACHABLE
        ]

    def test_service_notifications_options(self):
        contact = ContactDirective(self.stack, **self.requried_directive)
        assert ServiceNotifications.NO in contact.service_notifications_options

        self.requried_directive.pop('service_notifications_options')
        with self.assertRaises(TypeError):
            ContactDirective(self.stack, **self.requried_directive)

        self.requried_directive['service_notifications_options'] = [
            ServiceNotifications.WARNING,
            ServiceNotifications.FLAPPING
        ]
        contact = ContactDirective(self.stack, **self.requried_directive)
        assert contact.service_notifications_options == [
            ServiceNotifications.WARNING,
            ServiceNotifications.FLAPPING
        ]

    def test_synth(self):
        cg1 = ContactGroupDirective(
            self.stack, contactgroup_name='foo', alias='cg1')
        cg2 = ContactGroupDirective(
            self.stack, contactgroup_name='bar', alias='cg2')
        contact = ContactDirective(
            self.stack, contactgroups=[cg1, cg2], **self.requried_directive)
        tmp = (
            'define contact {',
            '    contact_name    foo-bar',
            '    contactgroups    foo,bar',
            '    host_notifications_enabled    1',
            '    service_notifications_enabled    1',
            '    host_notifications_period    24x7',
            '    service_notifications_period    24x7',
            '    host_notifications_options    d,u,r,f,s',
            '    service_notifications_options    n',
            '    host_notification_commands    email',
            '    service_notification_commands    email',
            '}')
        assert '\n'.join(tmp) == contact.synth()



import attr
from ndk.construct import Construct
from ndk.directives import *


@attr.s
class ContactGroupDirective(Construct):
    __object_type__ = 'contactgroup'

    contactgroup_name = PrimaryKey()
    alias = StringField(required=True)
    members = OneToMany('Contact')
    contactgroup_members = OneToMany('ContactGroup')

    @property
    def pk(self):
        return self.contactgroup_name



import attr
from ndk.construct import Construct
from ndk.directives import *


@attr.s
class HostGroupDirective(Construct):
    __object_type__ = 'hostgroup'

    hostgroup_name = PrimaryKey()
    alias = StringField(required=True)
    members = OneToMany('Host')
    hostgroup_members = OneToMany('HostGroup')
    notes = StringField()
    notes_url = StringField()
    action_url = StringField()

    @property
    def pk(self):
        return self.hostgroup_name



# Unit tests for Construct

import unittest
from collections.abc import Iterable

from ndk.construct import Construct
from ndk.stack import Stack


class ConstructTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('ConstructTesting')

    def test_construct(self):
        const = Construct(self.stack)
        assert const.stack is self.stack
        assert const.stack.name == 'ConstructTesting'

        const2 = Construct('new stack')
        assert const2.stack is not self.stack
        assert const2.stack.name == 'new stack'

    def test_object_type(self):
        assert Construct.__object_type__
        assert Construct.__object_type__ == 'template'

    def test_pk(self):
        const = Construct(self.stack)
        assert const.pk == 'template'

    def test_prxfix(self):
        const = Construct(self.stack)
        assert const.prefix == 'define template {'

    def test_suffix(self):
        const = Construct(self.stack)
        assert const.suffix == '}'

    def test_iter(self):
        const = Construct(self.stack)
        assert isinstance(const.__iter__(), Iterable)
        assert isinstance(const, Iterable)

    def test_synth(self):
        const = Construct(self.stack)
        tmp = ('define template {', '}')
        assert '\n'.join(tmp) == const.synth()



import unittest

from ndk.definitions import CommandDirective
from ndk.stack import Stack


class CommandDirectiveTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('commandTesting')

    def test_command_directive(self):
        with self.assertRaises(TypeError):
            CommandDirective(self.stack)
        with self.assertRaises(TypeError):
            CommandDirective(self.stack, command_name='foo')
        with self.assertRaises(TypeError):
            CommandDirective(self.stack, command_line='bar')
        assert CommandDirective(
            self.stack, command_name='foo', command_line='bar')

    def test_command_name(self):
        tp = CommandDirective(
            self.stack, command_name='Foo Bar', command_line='Foo Bar')
        assert tp.pk == 'foo-bar'
        assert tp.command_name == 'foo-bar'

    def test_command_line(self):
        tp = CommandDirective(
            self.stack, command_name='Foo Bar', command_line='Foo Bar')
        assert tp.command_line == 'Foo Bar'

    def test_synth(self):
        tp = CommandDirective(
            self.stack, command_name='Foo Bar', command_line='Foo Bar')
        tmp = (
            'define command {', '    command_name    foo-bar',
            '    command_line    Foo Bar', '}')
        assert '\n'.join(tmp) == tp.synth()



from enum import Enum


class ChoiceMixin:

    @classmethod
    def choices(cls):
        for name, member in cls.__members__.items():
            if not name == 'NO':
                yield member

    @classmethod
    def all(cls):
        return list(cls.choices())

    @classmethod
    def empty(cls):
        return [cls.NO]


class HostNotifications(ChoiceMixin, Enum):
    """
    This directive is used to define the host states for
    which notifications can be sent out to this contact.

    Valid options:
        - d = notify on DOWN host states
        - u = notify on UNREACHABLE host states
        - r = notify on host recoveries (UP states)
        - f = notify when the host starts and stops flapping
        - s = notify when host scheduled downtime starts and ends
        - n = the contact will not receive any type of host notifications
    """

    DOWN = 'd'
    UNREACHABLE = 'u'
    RECOVERY = 'r'
    FLAPPING = 'f'
    SCHEDULED = 's'
    NO = 'n'


class ServiceNotifications(ChoiceMixin, Enum):
    """
    This directive is used to define the service states for
    which notifications can be sent out to this contact.

    Valid options:
        - w = notify on WARNING service states
        - u = notify on UNKNOWN service states
        - c = notify on CRITICAL service states
        - r = notify on service recoveries (OK states)
        - f = notify when the service starts and stops flapping
        - n = the contact will not receive any type of service notifications.
    """

    WARNING = 'w'
    UNKNOWN = 'u'
    CRITICAL = 'c'
    RECOVERY = 'r'
    FLAPPING = 'f'
    SCHEDULED = 's'
    NO = 'n'




import attr
from ndk.definitions import contactgroup


@attr.s
class ContactGroup(contactgroup.ContactGroupDirective):
    alias = attr.ib(type=str,
                    converter=str,
                    validator=attr.validators.instance_of(str),
                    kw_only=True)

    @alias.default
    def _set_alias_as_contactgroup_name(self):
        return self.contactgroup_name



import attr
from ndk.definitions import service
from ndk.options import service as options


@attr.s
class Service(service.ServiceDirective):

    check_interval=attr.ib(default=3)
    retry_interval=attr.ib(default=2)
    check_period=attr.ib(default=5)
    notification_interval=attr.ib(default=60)
    max_check_attempts=attr.ib(default=5)
    active_checks_enabled=attr.ib(default=True)
    passive_checks_enabled=attr.ib(default=False)
    event_handler_enabled=attr.ib(default=False)
    flap_detection_enabled=attr.ib(default=True)
    notifications_enabled=attr.ib(default=True)



# Unit tests for Stack

import unittest
from collections import defaultdict
from collections.abc import Iterable

from ndk.construct import Construct
from ndk.stack import Stack


class StackTestCase(unittest.TestCase):

    def test_name(self):
        with self.assertRaises(TypeError):
            # `.name` is requried to initialize Stack
            Stack()
        stack = Stack('StackTesting')
        assert stack.name == 'StackTesting'

    def test_objects(self):
        stack = Stack('StackTesting')
        assert isinstance(stack.objects, defaultdict)

    def test_iter(self):
        stack = Stack('StackTesting')
        assert isinstance(stack.__iter__(), Iterable)
        assert isinstance(stack, Iterable)
        Construct(stack)
        assert len(list(stack)) == 1

    def test_singleton(self):
        # for .converter
        stack = Stack.singleton('StackTesting')
        new_stack = Stack.singleton(stack)
        assert stack is new_stack

    def test_push(self):
        stack = Stack('StackTesting')

        obj = type('foo', (object,), dict(__object_type__='foo', bar='baz'))
        with self.assertRaises(AssertionError):
            stack.push(obj)

        obj = Construct(stack)
        stack.push(obj)
        assert 'template' in stack.objects
        assert len(stack.objects['template']) == 1

    def test_synth(self):
        stack = Stack('StackTesting')
        Construct(stack)
        tmp = (
            'define template {',
            '}'
        )
        print(stack.synth())
        assert '\n'.join(tmp) == stack.synth()




import attr
from ndk.definitions import contact
from ndk.options import contact as options


@attr.s
class Contact(contact.ContactDirective):
    host_notifications_enabled = attr.ib(default=True)
    service_notifications_enabled = attr.ib(default=True)
    host_notifications_options = attr.ib(
        default=options.HostNotifications.all())
    service_notifications_options = attr.ib(
        default=options.ServiceNotifications.all())


@attr.s
class Email(Contact):
    # set email required
    email = attr.ib(type=str, converter=str, kw_only=True)



import unittest

import attr
from ndk.objects.command import Email as SendByEmail
from ndk.objects.contact import Contact, Email
from ndk.objects.timeperiod import TwentyFourSeven
from ndk.options import contact as options
from ndk.stack import Stack

# import HostNotifications, ServiceNotifications


class ContactTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('ContactTesting')

    def test_contact(self):
        tp = TwentyFourSeven(self.stack)
        cmd = SendByEmail(self.stack)
        contact = Contact(self.stack, contact_name='foo',
                          host_notifications_period=tp,
                          service_notifications_period=tp,
                          host_notification_commands=cmd,
                          service_notification_commands=cmd)
        assert contact.contact_name == 'foo'
        assert contact.host_notifications_enabled
        assert 'service_notifications_options    w,u,c,r,f,s' in contact.synth()


class EmailTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('EmailTesting')

    def test_email(self):
        tp = TwentyFourSeven(self.stack)
        cmd = SendByEmail(self.stack)
        with self.assertRaises(TypeError):
            contact = Email(self.stack, contact_name='foo',
                            host_notifications_period=tp,
                            service_notifications_period=tp,
                            host_notification_commands=cmd,
                            service_notification_commands=cmd)
        assert Email(self.stack, contact_name='foo', email='foo@bar.baz',
                     host_notifications_period=tp,
                     service_notifications_period=tp,
                     host_notification_commands=cmd,
                     service_notification_commands=cmd)



import attr
from ndk.construct import Construct
from ndk.directives import *


@attr.s
class TimePeriodDirective(Construct):
    __object_type__ = 'timeperiod'

    timeperiod_name = PrimaryKey()
    alias = StringField(required=True)
    sunday = StringField()
    monday = StringField()
    tuesday = StringField()
    wednesday = StringField()
    thursday = StringField()
    friday = StringField()
    saturday = StringField()

    @property
    def pk(self):
        return self.timeperiod_name



import attr
from ndk.definitions import command


@attr.s
class Command(command.CommandDirective):
    pass


@attr.s(frozen=True)
class Ping(Command):
    command_name = attr.ib(default='ping')
    command_line = attr.ib(
        default='$USER1$/check_ping -H $HOSTADDRESS$ -w 30.0,80% -c 50.0,100%')


@attr.s(frozen=True)
class Email(Command):
    command_name = attr.ib(default='send-by-email')
    command_line = attr.ib(default='smtp')



import unittest

from ndk.definitions import HostGroupDirective
from ndk.stack import Stack


class HostGroupDirectiveTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('HostGroupTesting')

    def test_contact_group_directive(self):
        HostGroupDirective(
            self.stack, hostgroup_name='foo', alias='bar')

    def test_hostgroup_members(self):
        cg_foo = HostGroupDirective(
            self.stack, hostgroup_name='foo', alias='FOO')
        cg_bar = HostGroupDirective(
            self.stack, hostgroup_name='bar', alias='BAR')
        contact = HostGroupDirective(
            self.stack, hostgroup_name='for-bar', alias='Foo Bar',
            hostgroup_members=[cg_foo, cg_bar])
        assert 'hostgroup_members    foo,bar' in contact.synth()



import attr
from ndk.definitions import hostgroup


@attr.s
class HostGroup(hostgroup.HostGroupDirective):
    alias = attr.ib(type=str,
                    converter=str,
                    validator=attr.validators.instance_of(str),
                    kw_only=True)

    @alias.default
    def _set_alias_as_hostgroup_name(self):
        return self.hostgroup_name



import unittest

import attr
from ndk.objects.command import Command, Email, Ping
from ndk.stack import Stack


class CommandTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('CommandTesting')

    def test_command(self):
        assert Command(
            self.stack, command_name='top', command_line='top')

    def test_command_name(self):
        with self.assertRaises(TypeError):
            Command(self.stack, command_line='bar')
        cmd = Command(
            self.stack, command_name='foo', command_line='bar')
        assert cmd.command_name == 'foo'

    def test_command_line(self):
        with self.assertRaises(TypeError):
            Command(self.stack, command_name='foo')
        cmd = Command(
            self.stack, command_name='foo', command_line='bar')
        assert cmd.command_line == 'bar'


class PingTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('PingTesting')

    def test_ping(self):
        assert Ping(self.stack)
        with self.assertRaises(attr.exceptions.FrozenInstanceError):
            cmd = Ping(self.stack)
            cmd.name = 'foo'

    def test_command_name(self):
        cmd = Ping(self.stack)
        assert cmd.command_name == 'ping'

    def test_command_line(self):
        cmd = Ping(self.stack)
        assert 'ping' in cmd.command_line


class EmailTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('EmailTesting')

    def test_email(self):
        assert Email(self.stack)
        with self.assertRaises(attr.exceptions.FrozenInstanceError):
            cmd = Ping(self.stack)
            cmd.name = 'foo'

    def test_command_name(self):
        cmd = Email(self.stack)
        assert cmd.command_name == 'send-by-email'

    def test_command_line(self):
        cmd = Email(self.stack)
        assert 'smtp' in cmd.command_line



from collections import defaultdict
# from ndk import construct
from importlib import import_module

import attr


class Stack:
    """A NDK stack.
    """

    def __init__(self, name):
        """Initialize a NDK stack.

        Args:
            name (str): the application name.
        """
        self.name = name
        self.objects = defaultdict(dict)

    def __iter__(self):
        """Iter all objects.

        Yields:
            obj (NagiosObject): the next obj in the range of the all objects.
        """
        for obj in self.objects.values():
            yield from obj.values()

    @classmethod
    def singleton(cls, stack):
        if isinstance(stack, cls):
            return stack
        return cls(stack)

    def push(self, obj):
        """Push a new Nagios Object to this stack.

        Args:
            obj (NagiosObject): The base class of Nagios.
        """
        construct = import_module('.construct', package='ndk')
        assert isinstance(
            obj, construct.Construct), f'{obj} must be Construct instance'
        self.objects[obj.__object_type__][obj.pk] = obj

    def synth(self):
        """Synthesizes the Nagios objects for this stack"""
        return '\n'.join((obj.synth() for obj in self))



import attr
from ndk.definitions import host
from ndk.options import host as options


@attr.s
class Host(host.HostDirective):
    max_check_attempts = attr.ib(default=5)
    notification_interval = attr.ib(default=60)
    active_checks_enabled = attr.ib(default=True)
    passive_checks_enabled = attr.ib(default=False)
    event_handler_enabled = attr.ib(default=False)
    flap_detection_enabled = attr.ib(default=True)
    notifications_enabled = attr.ib(default=True)
    notification_options = options.Notification.all()
    alias = attr.ib(type=str,
                    converter=str,
                    validator=attr.validators.instance_of(str),
                    kw_only=True)

    @alias.default
    def _set_alias_as_host_name(self):
        return self.host_name



import attr
from ndk.construct import Construct
from ndk.directives import *


@attr.s
class ServiceGroupDirective(Construct):
    __object_type__ = 'servicegroup'

    servicegroup_name = PrimaryKey()
    alias = StringField(required=True)
    members = OneToMany('Service')
    servicegroup_members = OneToMany('ServiceGroup')
    notes = StringField()
    notes_url = StringField()
    action_url = StringField()
 
    @property
    def pk(self):
        return self.servicegroup_name



import unittest

from ndk.definitions import ServiceGroupDirective
from ndk.stack import Stack


class ServiceGroupDirectiveTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('ServiceGroupTesting')

    def test_contact_group_directive(self):
        ServiceGroupDirective(
            self.stack, servicegroup_name='foo', alias='bar')

    def test_servicegroup_members(self):
        cg_foo = ServiceGroupDirective(
            self.stack, servicegroup_name='foo', alias='FOO')
        cg_bar = ServiceGroupDirective(
            self.stack, servicegroup_name='bar', alias='BAR')
        contact = ServiceGroupDirective(
            self.stack, servicegroup_name='for-bar', alias='Foo Bar',
            servicegroup_members=[cg_foo, cg_bar])
        assert 'servicegroup_members    foo,bar' in contact.synth()



from enum import Enum

import attr

from ndk.stack import Stack


@attr.s
class Construct(object):
    """A base class of Nagios Objects."""

    __object_type__ = 'template'
    stack = attr.ib(type=Stack,
                    converter=Stack.singleton,
                    validator=attr.validators.instance_of(Stack))

    def __attrs_post_init__(self):
        self.stack.push(self)

    @property
    def pk(self):
        return 'template'

    @property
    def prefix(self):
        return 'define %s {' % self.__object_type__

    @property
    def suffix(self):
        return '}'

    def synth(self):
        return "\n".join(self.__iter__())

    def __iter__(self):
        yield self.prefix
        # self.__dict__ has only attributes that created by attr.ib()
        for name, value in self.__dict__.items():
            if name == 'stack' or value is None:
                continue
            if isinstance(value, bool):
                value = 1 if value else 0
            if isinstance(value, list):
                if isinstance(value[0], Enum):
                    value = ','.join([v.value for v in value])
                else:
                    value = ','.join([str(v) for v in value if v])
            yield f'    {name}    {value}'
        yield self.suffix

    def __str__(self):
        return self.pk



import unittest

from ndk.objects import ContactGroup
from ndk.stack import Stack


class ContactGroupTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('ContactGroupTesting')

    def test_contact_group(self):
        cg = ContactGroup(self.stack, contactgroup_name='Foo')
        assert cg.pk == 'foo'
        assert cg.alias == 'foo'

    def test_alias(self):
        cg = ContactGroup(self.stack, contactgroup_name='foo', alias='bar')
        assert cg.contactgroup_name == 'foo'
        assert cg.alias == 'bar'





import attr
from ndk.construct import Construct
from ndk.directives import *
from ndk.options import contact as options


@attr.s
class ContactDirective(Construct):
    __object_type__ = 'contact'

    contact_name = PrimaryKey()
    alias = StringField()
    contactgroups = OneToMany('ContactGroup')
    minimum_importance = IntegerField()
    host_notifications_enabled = BooleanField(required=True)
    service_notifications_enabled = BooleanField(required=True)
    host_notifications_period = OneToOne(
        'TimePeriod', required=True)
    service_notifications_period = OneToOne(
        'TimePeriod', required=True)
    host_notifications_options = ChoiceField(
        options.HostNotifications, required=True)
    service_notifications_options = ChoiceField(
        options.ServiceNotifications, required=True)
    host_notification_commands = OneToOne('Command', required=True)
    service_notification_commands = OneToOne('Command', required=True)
    email = StringField()
    pager = StringField()
    addressx = StringField()
    can_submit_commands = BooleanField()
    retain_status_information = BooleanField()
    retain_nonstatus_information = BooleanField()

    @property
    def pk(self):
        return self.contact_name



import typing
from importlib import import_module

import attr
from attr import converters
from attr import ib as field
from attr import validators


class Converters:
    STR = converters.optional(str)
    INT = converters.optional(int)
    BOOL = converters.optional(bool)

    @staticmethod
    def PrimaryKey(name):
        return str(name).lower().replace(' ', '-')


def PrimaryKey():
    return field(type=str, converter=Converters.PrimaryKey)


def StringField(required=False):
    if required:
        return field(type=str, converter=str, kw_only=True)
    return field(
        type=str, converter=Converters.STR, default=None, kw_only=True)


def IntegerField(required=False):
    if required:
        return field(type=int, converter=int, kw_only=True)
    return field(
        type=int, converter=Converters.INT, default=None, kw_only=True)


def BooleanField(required=False):
    if required:
        return field(type=bool, converter=bool, kw_only=True)
    return field(
        type=bool, converter=Converters.BOOL, default=None, kw_only=True)


class ForeignKey:

    def __init__(self, relation):
        self.relation = relation

    @property
    def instance(self):
        module = import_module(f'ndk.definitions.{self.relation.lower()}')
        cls = getattr(module, self.relation+'Directive')
        return cls

    def member_validator(self, inst, attr, member):
        if isinstance(member, self.instance):
            return True
        return False


def OneToOne(relation, required=False):
    fk = ForeignKey(relation)
    if required:
        return field(
            type=type,
            validator=fk.member_validator,
            kw_only=True)
    return field(
        type=type,
        validator=validators.optional(fk.member_validator),
        default=None,
        kw_only=True)


def OneToMany(relation, required=False):
    fk = ForeignKey(relation)
    if required:
        return field(
            type=typing.List[type],
            validator=validators.deep_iterable(
                member_validator=fk.member_validator,
                iterable_validator=validators.instance_of(list),
            ),
            kw_only=True)
    return field(
        type=typing.List[type],
        validator=validators.optional(
            validators.deep_iterable(
                member_validator=validators.optional(fk.member_validator),
                iterable_validator=validators.instance_of(list),
            )),
        default=None,
        kw_only=True)


def ChoiceField(items, required=False):
    if required:
        return field(
            type=typing.List[items],
            validator=validators.deep_iterable(
                member_validator=validators.instance_of(items),
                iterable_validator=validators.instance_of(list))
        )
    else:
        return field(
            type=items,
            validator=validators.optional(
                validators.deep_iterable(
                    member_validator=validators.instance_of(items),
                    iterable_validator=validators.instance_of(list))
            ),
            default=None,
            kw_only=True)



from ipaddress import IPv4Address

import attr
from ndk.construct import Construct
from ndk.directives import *
from ndk.options import host as options


@attr.s
class HostDirective(Construct):
    __object_type__ = 'host'

    host_name = PrimaryKey()
    alias = StringField(required=True)
    display_name = StringField()
    address = attr.ib(
        type=IPv4Address, converter=attr.converters.optional(IPv4Address),
        default=None, kw_only=True)
    parents = OneToMany('Host')
    importance = IntegerField()
    hostgroups = OneToMany('HostGroup')
    check_command = OneToOne('Command')
    initial_state = ChoiceField(options.InitialState)
    max_check_attempts = IntegerField(required=True)
    check_interval = IntegerField()
    retry_interval = IntegerField()
    active_checks_enabled = BooleanField()
    passive_checks_enabled = BooleanField()
    check_period = OneToOne('TimePeriod', required=True)
    obsess_over_host = BooleanField()
    check_freshness = BooleanField()
    freshness_threshold = IntegerField()
    event_handler = OneToOne('Command')
    event_handler_enabled = BooleanField()
    low_flap_threshold = IntegerField()
    high_flap_threshold = IntegerField()
    flap_detection_enabled = BooleanField()
    flap_detection_options = ChoiceField(options.FlapDetection)
    process_perf_data = BooleanField()
    retain_status_information = BooleanField()
    retain_nonstatus_information = BooleanField()
    contacts = OneToMany('Contact')
    contact_groups = OneToMany('ContactGroup')
    notification_interval = IntegerField(required=True)
    first_notification_delay = IntegerField()
    notification_period = OneToOne('TimePeriod', required=True)
    notification_options = ChoiceField(options.Notification)
    notifications_enabled = BooleanField()
    stalking_options = ChoiceField(options.Stalking)
    notes = StringField()
    notes_url = StringField()
    action_url = StringField()
    icon_image = StringField()
    icon_image_alt = StringField()
    vrml_image = StringField()
    statusmap_image = StringField()
    # _2d_coords = StringField()
    # _3d_coords = StringField()

    @property
    def pk(self):
        return self.host_name



import attr
from ndk.construct import Construct
from ndk.directives import *


@attr.s
class CommandDirective(Construct):
    __object_type__ = 'command'

    command_name = PrimaryKey()
    command_line = StringField(required=True)

    @property
    def pk(self):
        return self.command_name



import unittest

import attr
from ndk.objects import BusinessDay, TimePeriod, TwentyFourSeven
from ndk.stack import Stack


class TimePeriodTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('TimePeriodTesting')

    def test_timeperiod(self):
        tp = TimePeriod(self.stack, timeperiod_name='foo')
        assert tp.pk == 'foo'
        assert tp.timeperiod_name == 'foo'
        assert tp.alias == 'foo'

    def test_weekdays(self):
        tp = TimePeriod(
            self.stack, timeperiod_name='foo', monday='00:00-24:00')
        assert tp.sunday is None
        assert tp.monday == '00:00-24:00'


class TwentyFourSevenTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('TwentyFourSevenTesting')

    def test_twenty_four_seven(self):
        tp = TwentyFourSeven(self.stack)
        assert tp.pk == '24x7'
        assert tp.alias == '24 hours per day, seven days per week'
        for day in [tp.sunday, tp.monday, tp.tuesday, tp.wednesday,
                    tp.thursday, tp.friday, tp.saturday]:
            with self.subTest(day=day):
                assert day == '00:00-24:00'

    def test_frozen(self):
        tp = TwentyFourSeven(self.stack)
        with self.assertRaises(attr.exceptions.FrozenInstanceError):
            tp.timeperiod_name = 'foo'


class BusinessDayTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('BusinessDayTesting')

    def test_business_day(self):
        tp = BusinessDay(self.stack)
        assert tp.pk == '8x5'
        assert tp.alias == '8 hours per day, from monday to friday'
        for day in [
                tp.monday, tp.tuesday, tp.wednesday, tp.thursday, tp.friday]:
            with self.subTest(day=day):
                assert day == '08:00-17:00'

    def test_frozen(self):
        tp = BusinessDay(self.stack)
        with self.assertRaises(attr.exceptions.FrozenInstanceError):
            tp.timeperiod_name = 'foo'



import unittest

from ndk.objects.command import Email
from ndk.objects.host import Host
from ndk.objects.timeperiod import TwentyFourSeven
from ndk.options import host as options
from ndk.stack import Stack


class HostTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('HostTesting')
        self.tp = TwentyFourSeven(self.stack)
        self.cmd = Email(self.stack)

    def test_host(self):
        host = Host(self.stack, host_name='Foo Bar',
                    check_period=self.tp, notification_period=self.tp)
        assert host.host_name == 'foo-bar'
        assert host.max_check_attempts == 5
        assert host.check_period.timeperiod_name == '24x7'

    # @unittest.skip('skip, the code is merging')
    # def test_L2_construct_is_works(self):
    #     _24x7 = objects.TwentyFourSeven(self.stack)
    #     cmd = objects.command.Ping(self.stack)  # fake cmd
    #     host = Host(self.stack, host_name='foo', address='localhost',
    #                 notification_period=_24x7, check_period=_24x7,
    #                 check_command=cmd)
    #     assert host.max_check_attempts == 5
    #     assert host.address == 'localhost'



import unittest

from ndk.definitions import TimePeriodDirective
from ndk.stack import Stack


class TimePeriodDirectiveTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('TimePeriodDirectiveTesting')

    def test_timeperiod_directive(self):
        with self.assertRaises(TypeError):
            TimePeriodDirective(self.stack)
        with self.assertRaises(TypeError):
            TimePeriodDirective(self.stack, timeperiod_name='foo')
        with self.assertRaises(TypeError):
            TimePeriodDirective(self.stack, alias='bar')
        assert TimePeriodDirective(
            self.stack, timeperiod_name='foo', alias='bar')

    def test_name(self):
        tp = TimePeriodDirective(
            self.stack, timeperiod_name='Foo Bar', alias='Foo Bar')
        assert tp.pk == 'foo-bar'
        assert tp.timeperiod_name == 'foo-bar'
        assert tp.alias == 'Foo Bar'

    def test_periods(self):
        tp = TimePeriodDirective(
            self.stack, timeperiod_name='Foo Bar', alias='Foo Bar',
            sunday='00:00-24:00')
        assert tp.sunday == '00:00-24:00'

    def test_synth(self):
        tp = TimePeriodDirective(
            self.stack, timeperiod_name='Foo Bar', alias='Foo Bar',
            sunday='00:00-24:00', monday='08:00-17:00')
        assert tp.monday == '08:00-17:00'
        tmp = (
            'define timeperiod {', '    timeperiod_name    foo-bar',
            '    alias    Foo Bar', '    sunday    00:00-24:00',
            '    monday    08:00-17:00', '}')
        assert '\n'.join(tmp) == tp.synth()



import unittest

from ndk.definitions import ContactGroupDirective
from ndk.stack import Stack


class ContactGroupDirectiveTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('ContactGroupTesting')

    def test_contact_group_directive(self):
        ContactGroupDirective(
            self.stack, contactgroup_name='foo', alias='bar')

    def test_contactgroup_members(self):
        cg_foo = ContactGroupDirective(
            self.stack, contactgroup_name='foo', alias='FOO')
        cg_bar = ContactGroupDirective(
            self.stack, contactgroup_name='bar', alias='BAR')
        contact = ContactGroupDirective(
            self.stack, contactgroup_name='for-bar', alias='Foo Bar',
            contactgroup_members=[cg_foo, cg_bar])
        assert 'contactgroup_members    foo,bar' in contact.synth()



from enum import Enum

from ndk.definitions import timeperiod
from ndk.directives import *


class WEEKDAYS(Enum):
    MONDAY = 'mon'
    TUESDAY = 'tue'
    WEDNESDAY = 'wed'
    THURSDAY = 'thu'
    FRIDAY = 'fri'


class WEEKEND(Enum):
    SUNDAY = 'sun'
    SATURDAY = 'sat'


@attr.s
class TimePeriod(timeperiod.TimePeriodDirective):
    alias = attr.ib(type=str,
                    converter=str,
                    validator=attr.validators.instance_of(str),
                    kw_only=True)

    @alias.default
    def _set_alias_as_timeperiod_name(self):
        return self.timeperiod_name


@attr.s(frozen=True)
class TwentyFourSeven(TimePeriod):
    timeperiod_name = attr.ib(type=str, default='24x7')
    alias = attr.ib(type=str, default='24 hours per day, seven days per week')

    sunday = attr.ib(default='00:00-24:00')
    monday = attr.ib(default='00:00-24:00')
    tuesday = attr.ib(default='00:00-24:00')
    wednesday = attr.ib(default='00:00-24:00')
    thursday = attr.ib(default='00:00-24:00')
    friday = attr.ib(default='00:00-24:00')
    saturday = attr.ib(default='00:00-24:00')


@attr.s(frozen=True)
class BusinessDay(TimePeriod):
    timeperiod_name = attr.ib(type=str, default='8x5')
    alias = attr.ib(type=str, default='8 hours per day, from monday to friday')

    monday = attr.ib(default='08:00-17:00')
    tuesday = attr.ib(default='08:00-17:00')
    wednesday = attr.ib(default='08:00-17:00')
    thursday = attr.ib(default='08:00-17:00')
    friday = attr.ib(default='08:00-17:00')



import unittest

from ndk.objects.command import Email, Ping
from ndk.objects.host import Host
from ndk.objects.service import Service
from ndk.objects.timeperiod import TwentyFourSeven
from ndk.options import service as options
from ndk.stack import Stack


class ServiceTestCase(unittest.TestCase):

    def setUp(self):
        self.stack = Stack('ServiceTesting')
        self.tp = TwentyFourSeven(self.stack)
        self.cmd = Email(self.stack)

    def test_service(self):
        host = Host(
            self.stack, host_name='foo', check_period=self.tp,
            notification_period=self.tp)
        ping = Ping(self.stack)
        assert Service(
            self.stack, host_name=host, service_description='bar',
            check_command=ping, notification_period=self.tp)

    def test_pk(self):
        host = Host(
            self.stack, host_name='foo', check_period=self.tp,
            notification_period=self.tp)
        ping = Ping(self.stack)
        srv = Service(
            self.stack, host_name=host, service_description='bar',
            check_command=ping, notification_period=self.tp)
        assert srv.pk == 'foo::bar'



from enum import Enum


class ChoiceMixin:

    @classmethod
    def choices(cls):
        for name, member in cls.__members__.items():
            if not name == 'NO':
                yield member

    @classmethod
    def all(cls):
        return list(cls.choices())

    @classmethod
    def empty(cls):
        return cls.NO


class InitialState(ChoiceMixin, Enum):
    """
    Override the initial state for a Host

    By default Nagios will assume that all hosts are in UP states when it starts.
    You can override the initial state for a host by using this directive.

    Valid options:
        - o = UP
        - d = DOWN
        - u = UNREACHABLE
    """

    UP = 'o'
    DOWN = 'd'
    UNREACHABLE = 'u'


class FlapDetection(ChoiceMixin, Enum):
    """
    This directive is used to determine what host states the flap detection logic will use for this host.

    Valid options:
        - o = UP
        - d = DOWN
        - u = UNREACHABLE
    """

    UP = 'o'
    DOWN = 'd'
    UNREACHABLE = 'u'


class Notification(ChoiceMixin, Enum):
    """
    This directive is used to determine when notifications for the host should be sent out.

    If you do not specify any notification options, Nagios will assume that you want notifications to be sent out for all possible states.

    Valid options:
        - d = send notifications on a DOWN state
        - u = send notifications on an UNREACHABLE state
        - r = send notifications on recoveries (OK state)
        - f = send notifications when the host starts and stops flapping
        - s = send notifications when scheduled downtime starts and ends
        - n = no host notifications will be sent out
    """

    DOWN = 'd'
    UNREACHABLE = 'u'
    RECOVERY = 'r'
    FLAPPING = 'f'
    SCHEDULED = 's'
    NO = 'n'


class Stalking(ChoiceMixin, Enum):
    """
    This directive determines which host states "stalking" is enabled for.

    Note: As of Core 4.4.0 you can use the N option to log event states when notifications are sent out.

    Valid options:
        - o = stalk on UP states
        - d = stalk on DOWN states
        - u = stalk on UNREACHABLE states
    """

    UP = 'o'
    DOWN = 'd'
    UNREACHABLE = 'u'
